---------------------------------------------------------------------------
-- Design unit:   Clock generator wrapper for Evariste II FPGA modules   --
---------------------------------------------------------------------------
-- File name:     gen_clk_wrp.vhd                                        --
---------------------------------------------------------------------------
-- System: VHDL'93                                                       --
---------------------------------------------------------------------------
-- Authors:                                                              --
-- Fischer V.                                                            --
-- (C) 2013 Laboratoire Hubert Curien                                    --
---------------------------------------------------------------------------
-- Description:   The block gen_clk_wrp ...                              --
---------------------------------------------------------------------------
-- Parameters:                                                           --
---------------------------------------------------------------------------
-- Inputs :                                                              --
--    * clk_in       : system clock                                      --
--                                                                       --
-- Outputs :                                                             --
--    * rnd_bito     : TRNG serial output (for oscilloscope)             --
--    * next_bito    : TRNG serial strobe (for oscilloscope)             --
--    * rnd_bito     : TRNG serial output (for oscilloscope)             --
--    * next_bito    : TRNG serial strobe (for oscilloscope)             --
--                                                                       --
---------------------------------------------------------------------------
-- Version: 0.0.1.01, May 2013                                           --
---------------------------------------------------------------------------
-- Revision history:                                                     --
-- Modifications in Version 1.01 made by VF in ...:                      --
--                                                                       --
-- Remarks:   -------------------------------------------------------------
-- It is necessary to make ifclk 5 ns earlier than clk.                  --
---------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_arith.ALL;

LIBRARY work;
USE work.lab_hc_pkg.ALL;

ENTITY gen_clk_wrp IS
PORT
  (
-- Input clock
  clk_in        : IN STD_LOGIC;  -- Input clock signal
  reset_pll     : IN STD_LOGIC; -- obligatoire pour cycloneV
  
-- Output clocks
  
  clk_ctrl      : OUT STD_LOGIC; -- and clk_usbif_int
  clk_applic    : OUT STD_LOGIC;
  clk_usbif_out : OUT STD_LOGIC;
  locked        : OUT STD_LOGIC 
 );
END gen_clk_wrp;

ARCHITECTURE rtl OF gen_clk_wrp IS

-------------------------------------------------------------------------------
-- Declarations of components
-------------------------------------------------------------------------------
-- Clock generator
--------------------------------------------------------------------------
-- pll1 generates 3 clocks:
-- c0 - clk_ctrl 
-- c1 - clk_core 
-- c2 - clk_usbif 
--------------------------------------------------------------------------
    COMPONENT pll1cycV IS
    PORT
    (
      refclk   : in  std_logic := '0'; --  refclk.clk
      rst      : in  std_logic := '0'; --   reset.reset
      outclk_0 : out std_logic;        -- outclk0.clk
      outclk_1 : out std_logic;        -- outclk1.clk
      outclk_2 : out std_logic;        -- outclk2.clk
      locked   : out std_logic         --  locked.export
    );
  END COMPONENT;
  
BEGIN
--  locked <= '1'; -- Only for Xilinx Spartan 3

-- Instantiation of components -----------------------------------------------
-- PLL1 instantiation  
  -- ifclk has to be in the range 5 - 48 MHz
  -- Use this pll to choose a frequency. Make ifclk ~6ns earlier than clk
i_pll1cycV: pll1cycV PORT MAP (
    refclk       => clk_in, 
    rst          => reset_pll,
    outclk_0     => clk_ctrl,      -- and clk_usbif_int (slow clock)
    outclk_2     => clk_applic,    -- and clk_rng (fast clock)
    outclk_1     => clk_usbif_out, -- clk_usbif_ind advanced by 6 ns
    locked => locked
  );
END rtl;